using System;
using System.Drawing;

namespace BackgroundCreator {

   /// <summary>
   /// Background types that can be displayed on each monitor.
   /// </summary>
   public enum BackgroundType {
      /// <summary>
      /// No image will be displayed, instead just showing the current background color.
      /// </summary>
      None,
      /// <summary>
      /// A random image will be displayed from the selected image directory and/or any of it's
      /// subdirectories
      /// </summary>
      Random,
      /// <summary>
      /// A specific Image will be displayed. The image must be specified or the background
      /// will not be created.
      /// </summary>
      SpecificImage
   }
   [Serializable]
   /// <summary>
   /// Information about each montior and the background that will be displayed.
   /// </summary>
   public class MonitorInformation {
      private static int monitorCount;

		public static void resetMonitorCount() {
			monitorCount = 1;
		}

      private int _monitorId;
      /// <summary>
      /// Id of the monitor. Primary monitor is 0.
      /// </summary>
      public int MonitorId {
         get { return _monitorId; }
      }

      private bool _isPrimary;
      /// <summary>
      /// Is this the primary monitor?
      /// </summary>
      public bool IsPrimary {
         get { return _isPrimary; }
      }

      private Rectangle _monitorDimensions;
      /// <summary>
      /// Dimensions of the monitor: Size is in relation to the resolution, position is relation to
      /// the primary monitor.
      /// </summary>
      public Rectangle MonitorDimensions {
         get { return _monitorDimensions; }
      }

      private String _specificImageFile = null;
      /// <summary>
      /// A specific image to draw on a monitor.
      /// </summary>
      public String SpecificImageFile {
         get { return _specificImageFile; }
         set { _specificImageFile = value; }
      }

      private BackgroundType _backgroundType = BackgroundType.Random;
      /// <summary>
      /// Which image should be displayed (None, Random, Specific)
      /// </summary>
      public BackgroundType BackgroundType{
         get { return _backgroundType; }
         set { _backgroundType = value; }
      }

      private Rectangle _configRectangle;
      /// <summary>
      /// The small rectangle displayed on the config screen.
      /// </summary>
      public Rectangle ConfigRectangle {
         get { return _configRectangle; }
         set { _configRectangle = value; }
      }

		private String _currentImageFile = null;
		public String CurrentImageFile {
			get { return _currentImageFile; }
			set { _currentImageFile = value; }
		}

      /// <summary>
      /// Constructor.
      /// </summary>
      /// <param name="dimensions">The dimensions of the monitor. 
      /// They will be in relation to the primary monitor not to the virtual screen.</param>
      /// <param name="isPrimary">Is the monitor primary. Must be determined in the enum because
      /// the working display area could be non-(0,0) if the taskbar is at the top.</param>
      public MonitorInformation(Rectangle dimensions, bool isPrimary) {
			_isPrimary = isPrimary;
         _monitorDimensions = dimensions;
         if (isPrimary) {
            _monitorId = 0;
         } else {
            _monitorId = monitorCount++;
         }
      }
   }
}
