using System;
using System.Collections.Generic;
using System.Text.RegularExpressions;
using System.Windows.Forms;
using System.Runtime.InteropServices;

using TrayRunnerFramework;

namespace MultiMonitor {
   [TrayRunnerFramework.TrayProgram("MultiMon", "app.ico")]
   [Serializable]
   /// <summary>
   /// Used to grab various pieces of data from the system for easy interaction.
   /// 
   /// Current grabs a URL from the clipboard, or blog data from the Windows Media Blogging Plugin.
   /// 
   /// </summary>
   public class MultiMon : MarshalByRefObject, ITrayProgram {

      private TrayMenuItem[] _trayMenu = new TrayMenuItem[0];
      public TrayMenuItem[] TrayMenu {
         get { return _trayMenu; }
      }

      private MultiMonProperties _props;

      public MultiMon() {
         _trayMenu = new TrayMenuItem[]{
														new TrayMenuItem("Toggle Secondary", new EventHandler(secondaryToggled)),
            new TrayMenuItem("Change Profile(s)", new EventHandler(changeProfiles))
			};

         _props = (MultiMonProperties)SaveProperties.LoadObject("MultiMon.options");
         if (_props == null) _props = new MultiMonProperties();

      }

      private void secondaryToggled(Object sender, EventArgs e) {
         if (_props.PrimaryOnly == false) {
            _props.Monitors = EnumAllMonitors();

            MonitorSettings[] monitors = new MonitorSettings[_props.Monitors.Count];
            _props.Monitors.CopyTo(monitors);
            foreach (MonitorSettings ms in monitors) {
               EnableDisplay(ms, ms.IsPrimary == true);
            }
         } else {
            foreach (MonitorSettings ms in _props.Monitors) {
               EnableDisplay(ms, true);
            }
            _props.Monitors.Clear();
         }
         _props.PrimaryOnly = !_props.PrimaryOnly;
      }

      private void changeProfiles(Object sender, EventArgs e) {
         if (_props.Profiles == null) _props.Profiles = new List<MultiMonitorProfile>();
         AddChangeProfiles acp = new AddChangeProfiles(_props.Profiles);
         acp.ShowDialog();
      }

      public void OnShutdown() {
         SaveProperties.SaveObject(_props, "MultiMon.options");
      }

      #region Statics

      internal static List<MonitorSettings> EnumAllMonitors() {
         DEVMODE dm = new DEVMODE();
         List<MonitorSettings> monitors = new System.Collections.Generic.List<MonitorSettings>();

         foreach (Screen s in System.Windows.Forms.Screen.AllScreens) {
            User32.EnumDisplaySettings(s.DeviceName, User32.ENUM_CURRENT_SETTINGS, ref dm);

            MonitorSettings m = new MonitorSettings(s.DeviceName, dm.dmDisplayFrequency, dm.dmBitsPerPel,
               dm.dmPelsHeight, dm.dmPelsWidth, Convert.ToInt16(dm.dmX), Convert.ToInt16(dm.dmY));
            monitors.Add(m);
         }

         return monitors;
      }

      internal static void EnableDisplay(MonitorSettings Monitor, bool enable) {
         DEVMODE dm = new DEVMODE();
         dm.dmDeviceName = new String(new char[32]);
         dm.dmFormName = new String(new char[32]);
         dm.dmSize = (short)Marshal.SizeOf(dm);

         if (dm.dmDisplayFrequency != Monitor.Frequency ||
            dm.dmBitsPerPel != Monitor.BitsPerPixel ||
            dm.dmPelsHeight != Monitor.Height ||
            dm.dmPelsWidth != Monitor.Width ||
            dm.dmX != Monitor.X ||
            dm.dmY != Monitor.Y) {

            dm.dmDisplayFrequency = Monitor.Frequency;
            dm.dmBitsPerPel = Monitor.BitsPerPixel;
            if (enable == true) {
               dm.dmPelsHeight = Monitor.Height;
               dm.dmPelsWidth = Monitor.Width;
            } else {
               dm.dmPelsHeight = 0;
               dm.dmPelsWidth = 0;
            }
            dm.dmX = Monitor.X;
            dm.dmY = Monitor.Y;

            dm.dmFields = User32.DM_BITSPERPEL | User32.DM_DISPLAYFREQUENCY | User32.DM_PELSHEIGHT | User32.DM_PELSWIDTH | User32.DM_POSITION;

            int result = User32.ChangeDisplaySettingsEx(Monitor.DisplayName, ref dm, IntPtr.Zero, User32.CDS_UPDATEREGISTRY, IntPtr.Zero);

            User32.EnumDisplaySettings(Monitor.DisplayName, User32.ENUM_CURRENT_SETTINGS, ref dm);
         }
      }
#endregion
      
      #region Interop
      [StructLayout(LayoutKind.Sequential)]
      public struct DEVMODE {
         [MarshalAs(UnmanagedType.ByValTStr, SizeConst = 32)]
         public string dmDeviceName;
         public short dmSpecVersion;
         public short dmDriverVersion;
         public short dmSize;
         public short dmDriverExtra;
         public int dmFields;
         public int dmX;
         public int dmY;
         public int reserved1;
         public int reserved2;
         /*
         public short dmXPosition;
         public short dmPaperSize;
         public short dmYPosition;
         public short dmPaperWidth;

         public short dmScale;
         public short dmCopies;
         public short dmDefaultSource;
         public short dmPrintQuality;
         */
         public short dmColor;
         public short dmDuplex;
         public short dmYResolution;
         public short dmTTOption;
         public short dmCollate;
         [MarshalAs(UnmanagedType.ByValTStr, SizeConst = 32)]
         public string dmFormName;
         public short dmLogPixels;
         public short dmBitsPerPel;
         public int dmPelsWidth;
         public int dmPelsHeight;

         public int dmDisplayFlags;
         public int dmDisplayFrequency;

         public int dmICMMethod;
         public int dmICMIntent;
         public int dmMediaType;
         public int dmDitherType;
         public int dmReserved1;
         public int dmReserved2;

         public int dmPanningWidth;
         public int dmPanningHeight;
      };

      class User32 {
         [DllImport("user32.dll")]
         public static extern int EnumDisplaySettings(string deviceName, int modeNum, ref DEVMODE devMode);
         [DllImport("user32.dll")]
         public static extern int ChangeDisplaySettings(ref DEVMODE devMode, int flags);

         [DllImport("user32.dll")]
         public static extern int ChangeDisplaySettingsEx(String deviceName, ref DEVMODE devMode,
            IntPtr hwnd, int dwFlags, IntPtr lParam);

         public const int ENUM_CURRENT_SETTINGS = -1;
         public const int CDS_UPDATEREGISTRY = 0x01;
         public const int CDS_TEST = 0x02;
         public const int DISP_CHANGE_SUCCESSFUL = 0;
         public const int DISP_CHANGE_RESTART = 1;
         public const int DISP_CHANGE_FAILED = -1;


         public const int DM_BITSPERPEL = 0x00040000;
         public const int DM_PELSWIDTH = 0x00080000;
         public const int DM_PELSHEIGHT = 0x00100000;
         public const int DM_DISPLAYFLAGS = 0x00200000;
         public const int DM_DISPLAYFREQUENCY = 0x00400000;
         public const int DM_POSITION = 0x00000020;
      }
      #endregion
   }
}