// TODO Save and restart state from a running instance
// TODO Unload/Reload hot keys against the new AppDomain

using System;
using System.IO;
using System.Reflection;
using System.Runtime.Remoting;
using System.Windows.Forms;

using System.Diagnostics;

using TrayRunnerFramework;

namespace TrayRunner {

   [Serializable]
   /// <summary>
   /// Class which tracks basic information about known domains.
   /// Also used to create the domain, and instantiate the primary class.
   /// </summary>
  public class AppDomainInformation {
      private AssemblyName _assemblyName;
      /// <summary>
      /// Stores the full name of the assembly for comparison purposes
      /// </summary>
      public AssemblyName AssemblyName {
         get { return _assemblyName; }
         set { _assemblyName = value; }
      }

      private String _name = null;
      /// <summary>
      /// Name of the application. This can be the application subdirectory or a name set through the [...] Attribute
      /// </summary>
      public String Name {
         get { return _name; }
         // TODO Find out where this is called and why
         set { 
            _name = value;
            setDomainState();
         }
      }

      private bool _isActive = false;
      /// <summary>
      /// Gets ir sets whether the domain is active. Setting this to false will actually unload the domain.
      /// </summary>
      public bool IsActive {
         get { return _isActive; }
         set { 
            if (value == _isActive) return;
            _isActive = value; 
            setDomainState();
         }
      }

      private FileInfo _assemblyFile = null;
      /// <summary>
      /// Sets the actual file name of the assembly for comparison purposes
      /// </summary>
      public FileInfo AssemblyFile {
         get { return _assemblyFile; }
         set { 
            _assemblyFile = value;
            _assemblyDirectory = _assemblyFile.DirectoryName;
            _assemblyFileName = _assemblyFile.Name.Substring(0, _assemblyFile.Name.Length - 
               _assemblyFile.Extension.Length);
         }
      }

      private String _typeName;
      public String TypeName {
         get { return _typeName; }
         set { _typeName = value; }
      }

      // Derived fields
      private String _assemblyDirectory = null;
      private String _assemblyFileName = null;

      /* Active or Deactive the domain based on set properties */

      private void setDomainState() {
         if (_assemblyFile == null && _name == null) return;
         
         if (_isActive == false) {
            if (_program != null) {
               try {
                  _program.OnShutdown();
                  AppDomain.Unload(_domain);
                  _domain = null;
                  _program = null;
                  if (DeactivateAppDomain != null) {
                     DeactivateAppDomain(this, EventArgs.Empty);
                  }
                  _fileSystemWatcher = null;
                  _fileSystemTimer.Stop();
                  _fileSystemTimer = null;

               } catch (Exception ex) {
                  Trace.WriteLine("EX:" + ex);
               }
            }
         } else {
            loadDomain();
            if (ActivateAppDomain != null) {
               ActivateAppDomain(this, EventArgs.Empty);
            }
            
            _fileSystemWatcher = new FileSystemWatcher(_assemblyDirectory);
            _fileSystemWatcher.EnableRaisingEvents = true;
            _fileSystemWatcher.Changed += new FileSystemEventHandler(_fileSystemWatcher_Changed);

         
            _fileSystemTimer = new System.Timers.Timer(1000);
            _fileSystemTimer.AutoReset = false;
            _fileSystemTimer.Elapsed += new System.Timers.ElapsedEventHandler(_fileSystemTimer_Elapsed);
         }      
      }

      /* Events */

      /// <summary>
      /// Fired when the app domain is activated.
      /// </summary>
      internal event EventHandler ActivateAppDomain;
      
      /// <summary>
      /// Fired when the app domain is deactivated.
      /// </summary>
      internal event EventHandler DeactivateAppDomain;
   
      // Instance-only information
      
      private AppDomain _domain;
      /// <summary>
      /// The actual domain the object is instantiated in.
      /// </summary>
      public AppDomain Domain {
         get { return _domain; }
      }

      private ITrayProgramLoader _program;
      /// <summary>
      /// The actual program as seen through the ITrayProgram Interface.
      /// </summary>
      public ITrayProgramLoader Program {
         get { return _program; }
      }

      private TrayMenuItem[] _trayMenuItems;
      /// <summary>
      /// Array of MenuItems. Pulled from the instantiated class.
      /// </summary>
      public TrayMenuItem[] TrayMenu {
         get { return _trayMenuItems; }
      }

     private PluginInformation _plugin;
     public PluginInformation Plugin {
        get { return _plugin; }
     }

      // Watch the directory we are loading
      FileSystemWatcher _fileSystemWatcher;
      
      // After a monitored change, wait the specified interval before taking any action
      // If another change is detected, reset the timer and watch again
      // This will avoid spurious reloads
      System.Timers.Timer _fileSystemTimer;

      /// <summary>
      /// Constructor. Takes a path to a plugin, and a boolean that determines if the plugin is active.
      /// If active is true, it then proceeds to load the domain and instantiate the primary class.
      /// </summary>
      /// <param name="applicationBase"></param>
      /// <param name="isActive"></param>
      public AppDomainInformation(FileInfo assemblyFileName, PluginInformation plugin) {
         this.Name = plugin.Name;
         this.AssemblyFile = assemblyFileName;
         this.IsActive = plugin.IsActive;

         this._plugin = plugin;
      }

		private void loadDomain() {
         _domain = ITrayProgramLoader.CreateDomain(_assemblyDirectory, _assemblyFileName, true);

         _program = ITrayProgramLoader.LoadPlugin(_domain, _assemblyFileName);

			_trayMenuItems = _program.MenuItems;

         _assemblyName = _program.AssemblyName;
		}

     	// We really only care if the dll we're watching changes
		private void _fileSystemWatcher_Changed(object sender, FileSystemEventArgs e) {
         if (_assemblyFile.Name.ToLower() == e.Name.ToLower() ) {
            _fileSystemTimer.Start();
         }
      }

      private void _fileSystemTimer_Elapsed(object sender, System.Timers.ElapsedEventArgs e) {
         // TODO Now that we know it's a changed file, we should check if the assembly name matches
         if (MessageBox.Show(String.Format("The Plugin '{0}' has changed. Would you like to reload?\r",
            _name), "Reload Plugin?",
            MessageBoxButtons.YesNo, MessageBoxIcon.Question, MessageBoxDefaultButton.Button1,
				MessageBoxOptions.ServiceNotification) == DialogResult.Yes) {
            
            this.IsActive = false;
            this.IsActive = true;
         }
		}
	}
}
