using System;
using System.Collections;
using System.IO;
using System.Reflection;
using System.Diagnostics;

using System.Runtime.Remoting;

namespace TrayRunnerFramework
{
	[Serializable]
	/// <summary>
	/// Used as a wrapper around the object to be run in the subdomain. Since this is loaded into the main domain
	/// It saves loading the plugin assemblies into the subdomain.
	/// </summary>
	public class ITrayProgramLoader : MarshalByRefObject	{
		private ITrayProgram _program;
		
		Hashtable _menuItemLookup;

      public static ITrayProgramLoader LoadPlugin(AppDomain ad, String name) {
         ITrayProgramLoader loader = createITrayProgramLoader(ad);
			loader.loadDll(name);

			return loader;
		}

      public static void ProbeAssembly(String applicationBase, String assemblyName, 
         out AssemblyName fullAssemblyName, out Hashtable plugins) {
         AppDomain ad = CreateDomain(applicationBase, assemblyName, false);

         ITrayProgramLoader loader = createITrayProgramLoader(ad);

         String name = assemblyName.Substring(0, assemblyName.Length - 4);
         loader.probeDll(name, out fullAssemblyName, out plugins);
      }

      private void probeDll(String name, out AssemblyName fullAssemblyName, out Hashtable plugins) {
         Assembly assm = AppDomain.CurrentDomain.Load(name);

         fullAssemblyName = assm.GetName();

         plugins = new Hashtable();

         foreach (Type t in assm.GetTypes() ) {
            Object[] attrs = t.GetCustomAttributes(typeof(TrayProgramAttribute), true);

            if (attrs.Length == 1) {
               TrayProgramAttribute attr = (TrayProgramAttribute)attrs[0];

               PluginInformation pi = new PluginInformation();
               pi.TypeName = t.Name;
               pi.Name = attr.ProgramName;
               pi.Description = attr.Description;

               if (attr.IconFileName != null) {
                  foreach (String resName in assm.GetManifestResourceNames()) {
                     if (resName.ToLower().EndsWith(attr.IconFileName.ToLower() ) ) {
                        Stream s = assm.GetManifestResourceStream(resName);
                        byte[] data = new byte[s.Length];
                        s.Read(data, 0, data.Length);
                        pi.PluginIcon = data;
                     }
                  }
               }

               plugins.Add(t.Name, pi);

               System.Diagnostics.Trace.WriteLine("Plugins:" + plugins.Count);
            }
         }
      }

      public static AppDomain CreateDomain(String applicationBase, String name, bool shadowCopyFiles) {
         AppDomainSetup ads = new AppDomainSetup();

         FileInfo fi = new FileInfo(applicationBase);
         ads.ApplicationName = name;
         
         ads.ApplicationBase = applicationBase;
         
         ads.CachePath = Path.Combine(
            Environment.GetFolderPath(Environment.SpecialFolder.LocalApplicationData), "TrayRunner");

         AppDomain ad = AppDomain.CreateDomain(name, null, ads);
         if (shadowCopyFiles) {
            ad.SetShadowCopyPath(applicationBase);
            ad.SetShadowCopyFiles();
         }

         return ad;
      }

      private static ITrayProgramLoader createITrayProgramLoader(AppDomain ad) {
         ObjectHandle oh = ad.CreateInstance(Assembly.GetExecutingAssembly().FullName,
            "TrayRunnerFramework.ITrayProgramLoader", false,
            BindingFlags.CreateInstance | BindingFlags.NonPublic | BindingFlags.Instance, 
            null, new Object[0] , null, null, null);

         ITrayProgramLoader loader = (ITrayProgramLoader)oh.Unwrap();

         return loader;
      }
		
		/// <summary>
		/// Private to avoid having the domain be created innacurately
		/// </summary>
		private ITrayProgramLoader() {
		}

		// TODO Instead of iterating through here, just use a type name
      private void loadDll(String dllName) {
			_menuItemLookup = new Hashtable();
			Assembly assm = AppDomain.CurrentDomain.Load(dllName);
         _assemblyName = assm.GetName();

			bool found = false;

			foreach (Type t in assm.GetTypes() ) {
				Object[] attrs = t.GetCustomAttributes(typeof(TrayProgramAttribute), true);

				if (attrs.Length == 1) {
					found = true;
					_program = (ITrayProgram)Activator.CreateInstance(t);

					_menuItems = new TrayMenuItem[_program.TrayMenu.Length];
					for (int i = 0; i < _menuItems.Length; i++) {
						_menuItemLookup.Add(_program.TrayMenu[i].Name, _program.TrayMenu[i].MenuEventHandler);
						TrayMenuItem newItem = new TrayMenuItem(_program.TrayMenu[i].Name,
							new EventHandler(menuEventHandler));
						_menuItems[i] = newItem;
					}
				}
			}
			if (found == false) throw new Exception("Not a TrayRunner plugin");
		}

		public void menuEventHandler(Object sender, EventArgs e) {
			String name = null;
			if (sender is System.Windows.Forms.MenuItem) {
				name = ((System.Windows.Forms.MenuItem)sender).Text;
			} else {
				name = (String)sender;
			}
			EventHandler eh = (EventHandler)_menuItemLookup[name];
			eh(this, e);
		}


		// According to IngoRammer, this will result in lifetime lease instead of just 5 minutes
		public override Object InitializeLifetimeService() {
			return null;
		}
		
		private TrayMenuItem[] _menuItems;
		public TrayMenuItem[] MenuItems {
			get { return _menuItems; }
		}

      private AssemblyName _assemblyName;
      public AssemblyName AssemblyName {
         get { return _assemblyName; }
         set { _assemblyName = value; }
      }

      private String _typeName;
      public String TypeName {
         get { return _typeName; }
         set { _typeName = value; }
      }

		public void OnShutdown() {
			_program.OnShutdown();
		}
	}
}
