// Note: This class is a cut & paste duplicate of SzurgotUtilities.SaveProperties, placed here for ease of use.
using System;
using System.Collections;
using System.IO;
using System.Runtime.Serialization;
using System.Runtime.Serialization.Formatters.Binary;

using System.Reflection;

namespace TrayRunnerFramework {
   public class SavePropertiesAttribute : System.Attribute {
   }

	/// <summary>
	/// Used to save options/properties from based on public Property Accessors. Create the options class as a subclass of SaveProperties,
	/// and it will be able to save a mutable class, allowing fields to be added/removed without interfering with serialization of
	/// existing versions.
	/// </summary>
	/// <remarks>
	/// This is a cut & paste copy of this class from SzurgotUtilities. It requires a filename to be specified due to
	/// GetEntryAssembly not quite working as planned.
	/// Note: For this to work, the subclass must have a constructor of the form:<br />
	/// <code>
	/// protected Options(System.Runtime.Serialization.SerializationInfo info, 
	/// System.Runtime.Serialization.StreamingContext context) : base(info, context) {
	/// }<br />
	/// </code>
	/// </remarks>
	[Serializable]
	public sealed class SaveProperties {
		
		private static String _defaultPath = null;
		/// <summary>
		/// Gets or sets the location for the configuration file. This will default to Environment.GetFolderPath(Environment.SpecialFolder.ApplicationData) with
		/// a subdirectory names by whatever is in the Assembly: AssemblyCompanyAttribute. I AssemblyCompany
		/// is not defined, or is an empty string, the subdirectory will be based off the Assembly.Name
		/// </summary>
		public static String DefaultPath {
			get {
				if (_defaultPath == null) {
					String appData = Environment.GetFolderPath(Environment.SpecialFolder.ApplicationData);
					Assembly assm = Assembly.GetExecutingAssembly();
					Object[] attrs = assm.GetCustomAttributes(typeof(AssemblyCompanyAttribute), true);

					String companyAttr = "";
					if (attrs.Length > 0) {
						companyAttr = ((AssemblyCompanyAttribute)attrs[0]).Company;
					}
					if (companyAttr == "") {
						companyAttr = assm.GetName().Name;
					}

					_defaultPath = Path.Combine(appData, companyAttr);
				}
				return _defaultPath;
			}
			set { _defaultPath = value; }
		}

		private static String _defaultFileName = null;
		/// <summary>
		/// Gets or sets the default name of the config file. Defaults to Assembly.GetEntryAssembly().GetName().Name + ".options".
		/// </summary>
		public static String DefaultFileName {
			get { 
				if (_defaultFileName == null) {
					Assembly assm = Assembly.GetEntryAssembly();
					_defaultFileName = Assembly.GetExecutingAssembly().GetName().Name + ".options";
				}
				return _defaultFileName;
			}
			set { _defaultFileName = value; }
		}

		private static String _defaultFullFileName = null;
		/// <summary>
		/// Gets or sets the Full Path & File Name for the options file. Defaults to
		/// <code>Path.Combine(DefaultPath, DefaultFileName);</code>
		/// </summary>
		public static String DefaultFullFileName {
			get { 
				if (_defaultFullFileName == null) {
					_defaultFullFileName = System.IO.Path.Combine(DefaultPath, DefaultFileName);
				}
				return _defaultFullFileName;
			}
			set { _defaultFullFileName = value; }
		}
		
		private SaveProperties () {
		}

		/// <summary>
		/// Saves the given object to the filename defined by DefaultPath & the given filename.
		/// </summary>
		/// <param name="objToSave"></param>
		/// <param name="fileName"></param>
		public static void SaveObject(Object objToSave, String fileName) {
			SaveObject(objToSave, DefaultPath, fileName);
		}

		/// <summary>
		/// This class will save the given object to given paht & filename.
		/// </summary>
		/// <param name="fileName"></param>
		/// <param name="objToSave"></param>
		public static void SaveObject(Object objToSave, String fullPath, String fileName) {
			String fullName = Path.Combine(fullPath, fileName);

			FileInfo fi = new FileInfo(fullName);
			Directory.CreateDirectory(fi.Directory.FullName);
			FileStream fs = new FileStream(fi.FullName, FileMode.Create);
			BinaryFormatter bf = new BinaryFormatter();
         bf.SurrogateSelector = new SurrogateSelector();
			bf.Binder = new Binder();
			bf.Serialize(fs, objToSave);
			fs.Close();
		}

		public static Object LoadObject(String fileName) {
			return LoadObject(DefaultPath, fileName);
		}
		
		/// <summary>
		/// This class will load a given object from a given filename.
		/// </summary>
		/// <param name="fileName"></param>
		/// <returns></returns>
		public static Object LoadObject(String filePath, String fileName) {
			String name = Path.Combine(filePath, fileName);
			try {
				FileStream fs = new FileStream(name, FileMode.Open);
				BinaryFormatter bf = new BinaryFormatter();
				bf.SurrogateSelector = new SurrogateSelector();
				bf.Binder = new Binder();
				Object o = bf.Deserialize(fs);
				fs.Close();
				return o;
			} catch (System.IO.FileNotFoundException) {
			} catch (SerializationException ex) {
				System.Diagnostics.Trace.WriteLine(ex.ToString() );
				System.Diagnostics.Trace.WriteLine("Deserialization Error. Is the constructor in place in the subclass?");
			} catch (IOException ioe) {
				System.Diagnostics.Trace.WriteLine("IO Exception:" + ioe);
			}
			return null;
		}
	}
}
